-- File-Revision: 15
-- cooldown stuff

local find = string.find
local tremove = table.remove;
local tinsert = table.insert;

RCOptions = {};

local defaults = {disableBuff = false, disableDebuff = false};

local numTimers = 0;
local unusedTimers = {};
local activeCooldowns = {};

function cooldownTimer_PrintDebug()
  DEFAULT_CHAT_FRAME:AddMessage("CooldownTimer Debug stuff");
  DEFAULT_CHAT_FRAME:AddMessage("numTimers: "..numTimers);
  DEFAULT_CHAT_FRAME:AddMessage("unusedTimers: "..#unusedTimers);
  
  for i=1,numTimers do 
    DEFAULT_CHAT_FRAME:AddMessage("timer #"..i.." has parent "..((_G["CooldownTimers"..i]:GetParent():GetName()) or "nil"));
  end
end

local function getCooldownTimer(cooldown)
  local timer = tremove(unusedTimers);
  if ( not timer ) then
    numTimers = numTimers + 1;
    timer = CreateFrame("FRAME","CooldownTimers"..numTimers,RangeCooldownFrame,"CooldownTimerTemplate");
  end
   
  return timer
end

local function freeCooldownTimer(cooldown)
  local timer = cooldown.timer;
  timer:Hide();
  timer:SetParent(RangeCooldownFrame);
  
  tinsert(unusedTimers, timer);
  cooldown.timer = nil;
end

SLASH_RANGECOOLDOWN1, SLASH_RANGECOOLDOWN2 = '/rangecooldown', '/rc'
function SlashCmdList.RANGECOOLDOWN(msg, editbox)
  local command, rest = msg:match("^(%S*)%s*(.-)$");
  
  if command == "buff" then
    if rest == "show" then
      RCOptions.disableBuff = false;
      RCPrintMessage("Buff Cooldowns Enabled");
    else
      RCOptions.disableBuff = true;
      RCPrintMessage("Buff Cooldowns Disabled");
    end  
  elseif command == "debuff" then
    if rest == "show" then
      RCOptions.disableDebuff = false;
      RCPrintMessage("Debuff Cooldowns Enabled");
    else
      RCOptions.disableDebuff = true;
      RCPrintMessage("Debuff Cooldowns Disabled");
    end
  else
    RCPrintMessage("Syntax: /rangecooldown (buff|debuff) (show||hide)");
  end
end

function RCPrintMessage(msg)
  DEFAULT_CHAT_FRAME:AddMessage("|cFF00FF00RangeCooldown: |r"..msg, .2, .4, 1.0);
end

-- updates the timer text if there is some time left
function cooldownTimer_OnUpdate(self, elapsed)
  self.elapsed = self.elapsed + elapsed;
  if self.elapsed > .1 then
    cooldownTimer_SetTime(self);
--    cooldownTimer_CheckTimer(self);
    self.elapsed = 0;
    if self.time == 0 then
      local cooldown = self:GetParent();
      freeCooldownTimer(cooldown);
    end
--    if self.check then
--      local action = ((self:GetParent()):GetParent()).action
--      cooldownTimer_CheckTimer(self, action);
--    end
  end
end

-- Formats and then sets the time on the given timer
function cooldownTimer_SetTime(timer)
  local timeleft = timer.time - GetTime();

  if timeleft < 0 then
    timer.time = 0;
  elseif timeleft < 1 then
    timer.text:SetFontObject("CooldownFontSmall");
    timer.text:SetText(("%.1f"):format(timeleft));
  elseif timeleft < 60 then
    timer.text:SetFontObject("CooldownFontLarge");
    timer.text:SetText(("%d"):format(timeleft));
  elseif timeleft < 3600 then
    timeleft = floor(timeleft/60);
    timer.text:SetFontObject("CooldownFontMedium");
    timer.text:SetText(("%dm"):format(timeleft));
  else
    timeleft = floor(timeleft/3600);
    timer.text:SetFontObject("CooldownFontLarge");
    timer.text:SetText(("%dh"):format(timeleft));
  end
end

-- function to make sure the actionbuttons have timers created for them
function cooldownTimer_UpdateAction(button)
  local timer = _G[button:GetName().."CooldownTimer"]
  timer = timer or cooldownTimer_CreateTimer(button.cooldown);
   if button.action and HasAction(button.action) then
    cooldownTimer_CheckTimer(timer, action);
    timer:Show();
  else
    timer:Hide();
  end
end

-- funtion hooked to the set timer function for cooldowns.  gets called for items and most cooldowns except action buttons.
function cooldownTimer_SetTimer(self, start, duration, enable, charges, maxcharges)
  --local button = self:GetParent();
  local name = self:GetName(); 

  -- grab the timer object for the given button
--  local timer = _G[name.."Timer"];

  if (RCOptions.disableBuff and find(name,'Buff')) or (RCOptions.disableDebuff and find(name,"Debuff")) then
    if self.timer then
      freeCooldownTimer(self);
    end
    return;
  end

   -- make sure we have valid timer, create one if necessary
  --timer = timer or cooldownTimer_CreateTimer(self);
 
  if not self:GetParent():IsShown() then
    return
  end

  -- if we have a start time and the duration is over the base gcd of 1.5 then set the time
  if (start and start > 0 and duration > 1.5 and enable > 0) then
    if not self.timer then
      self.timer = getCooldownTimer(self);

      if (find(name,"Buff") or find(name,"Debuff")) then
        self.timer:SetScale(.5);
      else
        self.timer:SetScale(1.0);
      end
      
      self.timer:SetParent(self);
      self.timer:ClearAllPoints();
      self.timer:SetAllPoints();
      
      -- default to 0
      self.timer.elapsed = 0;
    end
    self.timer.time = duration + start;
    self.timer:Show();
    cooldownTimer_SetTime(self.timer);
  else
    if self.timer then
      freeCooldownTimer(self);
    end
  end
end

-- Functions to keep track of active cooldowns on action buttons
local function cooldown_OnShow(self)
  activeCooldowns[self] = true;
end

local function cooldown_OnHide(self)
  activeCooldowns[self] = nil;
  if self.timer then
    freeCooldownTimer(self);
  end
end

function cooldownTimer_AddAction(button, action, cooldown)
  if not cooldown.timerAction then
    cooldown:HookScript('OnShow', cooldown_OnShow);
    cooldown:HookScript('OnHide', cooldown_OnHide);
  end
  cooldown.timerAction = action;
end

-- function to check all active for cooldowns which match our requirements
function cooldownTimer_CheckCooldowns()
  local i, cooldown;
  for cooldown in pairs(activeCooldowns) do
    local start, duration, enable, charges, maxcharges = GetActionCooldown(cooldown.timerAction);
    cooldownTimer_SetTimer(cooldown, start, duration, enable, charges, maxcharges);
  end
end

do 
  --hooksecurefunc("ActionButton_Update",cooldownTimer_UpdateAction);
  hooksecurefunc("SetActionUIButton",cooldownTimer_AddAction);
  hooksecurefunc("CooldownFrame_SetTimer",cooldownTimer_SetTimer);
end

-- Create a timer frame for the given cooldown frame
function cooldownTimer_CreateTimer(self)
  local frameName = self:GetName().."Timer";
  
  local frame = CreateFrame("FRAME",frameName,self,"CooldownTimerTemplate");
  
  if (self:GetParent()).cooldown then
    frame:RegisterEvent("ACTIONBAR_UPDATE_STATE");
  end
  
  frame.text:SetText("");
  frame.text:SetTextColor(1.0,0.1,0.1);
  
  frame.time = 0;
  frame.elapsed = 0;
  
    -- if we are a debuff lets make the text smaller to fit in the smaller icon
  if find(frameName,'Buff') or find(frameName,"Debuff") then
    frame:SetScale(.5);
  end
  
  self.timer = frame;
  
  return frame;
end

-- When a cooldown frame is shown check for the cooldown that needs to be shown
function cooldownTimer_OnShow(self, ...)
  --self.check = true;
end

-- function called when the actionbar cooldown infomation has been updated
function cooldownTimer_OnEvent(self, event, arg1)
  local action = ((self:GetParent()):GetParent()).action
  cooldownTimer_CheckTimer(self, action);
end

-- Checks the given timer to see if the action given has a cooldown that needs to be shown
function cooldownTimer_CheckTimer(timer, action)

  -- if there is an action sent in and its has a valid action attached
  if action and HasAction(action) then
    local start, duration, enable, charges, maxcharges = GetActionCooldown(action);
    
    -- if the cooldown is enabled and has a valid start
    if enable and enable > 0 and start and start > 0 then
      -- if the duration is over 1.5s then show the text
      if duration > 1.5 then
        timer.time = duration + start;
        timer.text:Show();
        cooldownTimer_SetTime(timer);
      else
        -- otherwise hide the text
        timer.text:Hide();
      end
      
      -- we found the cd so no need to check more
      timer.check = nil;
    else
      -- no cooldown so hide text
      timer.text:Hide();
    end
  end
end

function cooldownTimerFrame_OnEvent(self, event, arg1)
  
  if event == "ACTIONBAR_UPDATE_COOLDOWN" then
    cooldownTimer_CheckCooldowns();
    return;
  end
  
  if event == "PLAYER_LOGIN" then
    local i, button;
    
    -- Add all the current actions to the list
    for i, button in pairs(ActionBarButtonEventsFrame.frames) do
      cooldownTimer_AddAction(button, button.action, button.cooldown);
    end
    
    cooldownTimer_CheckCooldowns();
    
    self:UnregisterEvent("PLAYER_LOGIN");
    return
  end
  
  if event == "ADDON_LOADED" then
    if arg1 == "RangeCooldown" then
      -- load our defaults if no settings are found
      local key, value;
      for key, value in pairs(defaults) do
        if RCOptions[key] == nil then
          RCOptions[key] = value;
        end
      end      
      self:UnregisterEvent("ADDON_LOADED");
    end
    return;
  end

end

